
// ATmega16 Timer1 Zeitmessung im Capture-Betrieb
// Port A: Ausgabe Capture_High BCD Tausender Hunderter 
// Port B: Ausgabe Capture_Low  BCD Zehner    Einer
// Port C: Display-Ausgabe
// Port D: Eingabe PD6 = ICP1 
// 8 MHz : 1024 = 128 us / Anzeigeeinheit max. 9999 * 128 us = 1.28 sek
// 
// Das Programm misst die Zeit zwischen fallender uns steigender Flanke an "PD6"
// __________                     __________
//           |___________________|
//           <--- Zeitmessung --->

#include <avr/io.h>               					     // Deklarationen 
#include <avr/signal.h>            					     // fr Interrupt
#include <avr/interrupt.h>       					     // fr Interrupt
#include "lcd-routines.h"                                // Header-File aus dem AVR-GCC-Tutorial 
                                                         // Header-File gendert (LCD sitzt an Port C)
#include <stdlib.h>

volatile uint16_t wert;

SIGNAL(SIG_INPUT_CAPTURE1)       					     // Interrupt Capture 
{
 // register unsigned int wert;    					     // Timertakte in ICR1
 unsigned char flag;
 wert = ICR1;                    					     // Capture-Wert auslesen
 flag = TIFR;                     					     // Timer1 berlaufmarke lesen  
 if( (wert > 9999) | (flag & (1 << TOV1)))  		     // berlauf
 {
  PORTA = 0xaa; 									     // Fehlermarke < <
  PORTB = 0xaa; 									     // Fehlermarke < <
  TIFR = flag;  									     // Flag lschen 
 }
 else
 {
  PORTA = (wert/1000 << 4) | (wert%1000)/100; 		     // Tausender und Hunderter
  wert %= 100; 
  PORTB = (wert/10 << 4) | wert%10;          		     // Zehner und Einer 
  
 } 													     // Ende if-else
} 													     // Ende Servicefunktion

// --- Hauptprogramm --- 


float periode;                                           // Variable zum umrechnen von wert --> Periodendauer 
float drehzahl;                                          // Variable zum berechnen der Drehzahl
void main(void)            							     // Hauptfunktion 
{
lcd_init();                                              // LCD-Initialisirung
set_cursor(0,1);                                         // Gehe zu Pos. XY
lcd_string("Drehzahlmesser" );                           // Ausgabe Maske
set_cursor(0,2);                                         // Gehe zu Pos. XY
lcd_string("U/min:" );                                   // Ausgabe Maske

 DDRA = 0xff;                						     // Port A ist Ausgabe
 DDRB = 0xff;                                            // Port B ist Ausgabe
              						 
 TIMSK  |= (1 << ICF1);     						     // ICF1 Capture Interrupt frei
 TCCR1B |= (1 << ICES1) | (1 << CS12) | (1 << CS10);     // steig. Flanke Teiler :1024
 sei();                     						 	 // Interrupts frei
 while(1) 
 { 
  while(PIND & (1 << PD6));      						 // warte solange PD6 High
  TCNT1 = 0;                     					     // Timer1 lschen Messung beginnt
  while( !(PIND & (1 << PD6))); 						 // warte solange PD6 Low
  
// --- Berechnung und Ausgabe ---
periode=wert*0.000128;                                   // Berechnung der Periodendauer

if (wert==0)                                             // Ausschluss unmglicher Rechenoparationen 
                                                         // (division durch 0 ist nicht deffiniert)
{ 
 drehzahl=0;                                             // fals wert=0 --> Drehzahl = 0
}
else
{
 drehzahl=60/periode;                                    // Berechnung der Umdrehungen in U/min
}
lcd_clear();           
set_cursor(0,1);                                         // Gehe zu Pos. XY
lcd_string("Drehzahlmesser" );                           // Ausgabe Maske
set_cursor(0,2);                                         // Gehe zu Pos. XY
lcd_string("U/min:" );                                   // Ausgabe Maske

   set_cursor(8,2);                                      // Gehe an POS. XY
  char Buffer [20];                              
  itoa( drehzahl, Buffer, 10 );                          // Umwandlung
  lcd_string( Buffer );                                  // Ausgabe Drehzahl
 } 												    	 // Ende while          
} 													     // Ende main





